from dataclasses import dataclass, field
from typing import List, Optional
from datetime import datetime
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.dossier_parametrage.recupStructuresResponse import UniteOrganisationnelleDP
from wssiham.model.parametre.uo import ParamUO


import zeep
import logging

logger = logging.getLogger(__name__)


@dataclass
class RecupStructures(Zeepclient):
    """ Ce flux a pour objectif d’envoyer toutes les informations pertinentes sur les unités
    organisationnelles dans SIHAM.
    """
    liste_uo: Optional[List[ParamUO]] = field(default=None)
    code_administration: Optional[str] = field(default=None)        # 3 caractères (exemple: 'RUN' pour Université Réunion)
    date_observation: Optional[datetime] = field(default=None)      # format YYYY-MM-DD-HH.MM.SS

    def __post_init__(self):
        super().__post_init__()
        if self.liste_uo is None and self.code_administration is None and \
                self.date_observation is None:
            raise TypeError("Valeur requise pour au moins un champ.")

    def response(self) -> Optional[list[UniteOrganisationnelleDP]]:
        """ Récuperer les données en vigueur à une date donnée pour une unité organisationnelle

        Returns:
            Liste contenant les informations sur les données d'une ou des unités organisationnelles

        """
        result = None
        try:
            resp = \
                self.zeep_client.service.RecupStructures(ParamStructure=
                                                         {'dateObservation': self.date_observation,
                                                          'codeAdministration': self.code_administration,
                                                          'listeUO': self.liste_uo})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [UniteOrganisationnelleDP(**ele) for ele in tmp_data]
                logger.debug(tmp_data)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result


