from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional
from wssiham.connection.zeepclient import Zeepclient

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class Indicateur(Zeepclient):
    code_etablissement: str = field(default='')                 # code de l'établissement (exemple : 9740478B)
    code_uo_affectation: str = field(default='')                # code UO
    temoin_uo_fille: str = field(default='0')                   # 0 ou 1
    nom_patronymique: str = field(default='')
    nom_usuel: str = field(default='')
    prenom: str = field(default='')
    date_observation: Optional[datetime] = field(default=None)              # format YYYY-MM-DD-HH.MM.SS

    def response(self) -> int:
        """ Récupère le nombre d’agents de la population répondant aux critères de la sélection.

        Returns:
            Le nombre d'agents trouvé correspondant aux critres de sélection
        """
        result = -1
        try:
            param = {'codeEtablissement': self.code_etablissement,
                     'codeUOAffectation': self.code_uo_affectation,
                     'dateObservation': self.date_observation,
                     'nomPatronymique': self.nom_patronymique,
                     'nomUsuel': self.nom_usuel,
                     'prenom': self.prenom,
                     'temoinUOFille': self.temoin_uo_fille
                     }
            resp = self.zeep_client.service.RecupererNbrIndividus(
                ParamIndicateur=param)
            if type(resp) == int:
                logger.debug(f"Liste des paramètres utilisés pour ParamIndicateur \n"
                             f"{param}")
                result = resp
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result


