from dataclasses import dataclass, field
from typing import List, Union
from datetime import datetime
from typing import Optional

from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_agents.genericAgentsReponse import InformationAgentEcheanceLA

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupAgentsEcheance(Zeepclient):
    """Ce Web Service retourne les matricules des agents dont l’un des attributs échangés,
    gérés par date, est entré en vigueur ou arrive à échéance entre les deux dates données en paramètre.
    """

    date_debut_observation: Optional[datetime] = field(default=None)              # format YYYY-MM-DD-HH.MM.SS
    date_fin_observation: Optional[datetime] = field(default=None)              # format YYYY-MM-DD-HH.MM.SS
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[list[InformationAgentEcheanceLA]]:
        """ Retourne les codes des structures dont l’un des attributs échangés entre en vigueur ou
         arrive à échéance entre les deux dates données


        Returns:
            Liste contenant les informations concernant le matricule, les témoins et le login Ldap de l'agent.
            (matricule, témoin état de l'agent, témoin retraite, UID)

        """
        result = None
        try:

            resp = self.zeep_client.service.RecupAgentsEcheance(
                ParamAgentEcheance={'dateDebutObservation': self.date_debut_observation,
                                    'dateFinObservation': self.date_fin_observation})

            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [InformationAgentEcheanceLA(**ele) for ele in tmp_data]
                self.found = (True, len(result))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result
