from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional, List
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_agents.genericAgentsReponse import InformationAgenModifiesLA

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupAgentsModifies(Zeepclient):
    """ Le Web service des agents modifiés est développé pour répondre aux besoins de l’interface PRISME et de
    l’interface LDAP. Il permet l’extraction des matricules SIHAM des agents dont l’un des attributs échangés avec Ldap
    ou Prisme a été modifié par l’utilisateur après la date et heure d’observation passée en paramètre
    """

    date_observation: Optional[datetime] = field(default=None)
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[List[InformationAgenModifiesLA]]:
        """ Retourne les matricules des agents dont l’un des attributs échangés a été modifié

        Returns:
            Liste contenant les informations concernant le matricule et le login Ldap de l'agent
            (matricule, userid)

        """
        result = None
        try:
            resp = \
                self.zeep_client.service.RecupAgentsModifies(DateHeureDebObservation=self.date_observation)
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [InformationAgenModifiesLA(**ele) for ele in tmp_data]
                self.found = (True, len(result))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

