from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_parametrage.recupNomenclaturesModifieesResponse import NomenclatureModifie

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupNomenclaturesModifiees(Zeepclient):
    """ Le Web service des nomenclatures modifiées est développé pour répondre aux besoins de l’interface PRISME et
    de l’interface LDAP. Il permet l’extraction des codes SIHAM des nomenclatures dont l’un des attributs échangés avec
    Ldap ou Prisme (Flux - Envoi des nomenclatures) a été modifié par l’utilisateur depuis la dernière date/heure
    d’observation passée en paramètre.
    """

    date_observation: Optional[datetime] = field(default=None)
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[List[NomenclatureModifie]]:
        """ Retourne les codes des nomenclatures dont l’un des attributs échangés a été modifié

        Returns:
            Liste contenant les informations concernant le (code répertoire, code information)

        """
        result = None
        try:
            resp = \
                self.zeep_client.service.RecupNomenclaturesModifiees(DateHeureDebObservation=self.date_observation)
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [NomenclatureModifie(**ele) for ele in tmp_data]
                self.found = (True, len(result))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

