from dataclasses import dataclass, field
from typing import List, Optional
from datetime import datetime
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_parametrage.genericStructureResponse import StructureEcheanceLP

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupStructuresEcheance(Zeepclient):
    """ Ce WebService retourne les codes des structures organisationnelles dans SIHAM dont l’un des attributs échangés
    (Flux - Envoi des structures), gérés par date, entre en vigueur ou arrive à échéance entre les deux dates données
    """

    date_debut_observation: Optional[datetime] = field(default=None)
    date_fin_observation: Optional[datetime] = field(default=None)
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[List[StructureEcheanceLP]]:
        """ Retourne les codes des structures dont l’un des attributs échangés entre en vigueur ou
         arrive à échéance entre les deux dates données


        Returns:
            Liste contenant les informations concernant le (code structure, code Information, type UO, témoin État)

        """
        result = None
        try:

            resp = \
                self.zeep_client.service.RecupStructuresEcheance(
                    ParamStructures={'dateDebutObservation': self.date_debut_observation,
                                     'dateFinObservation': self.date_fin_observation})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [StructureEcheanceLP(**ele) for ele in tmp_data]
                self.found = (True, len(result))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

