from dataclasses import dataclass, field
from typing import List, Optional
from datetime import datetime
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.response.liste_parametrage.genericStructureResponse import StructureModifieLP

import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupStructuresModifies(Zeepclient):
    """ Le Web service des structures modifiées est développé pour répondre aux besoins de l’interface PRISME et de
    l’interface LDAP. Il permet l’extraction des codes SIHAM des structures dont l’un des attributs échangés avec
    Ldap ou Prisme (Web service Envoi des structures) a été modifié par l’utilisateur
    depuis la dernière date/heure d’observation passée en paramètre.
    """

    date_observation: Optional[datetime] = field(default=None)
    found: tuple[bool, int] = \
        field(init=False, default=(False, -1), repr=False)  # tuple déterminant si un élément a été trouvé et son nombre

    def response(self) -> Optional[List[StructureModifieLP]]:
        """ Retourne les codes des structures dont l’un des attributs échangés a été modifié
        - Libellés réglementaires
        - Identifications des UO
        - Libellés des UO
        - Historique des situations
        - Identification du responsable
        - Adresses
        - Numéros de téléphones
        - Historique liens organisationnels
        - Etablissement


        Returns:
            Liste contenant les informations concernant le (code structure, type UO)

        """
        result = None
        try:
            resp = \
                self.zeep_client.service.RecupStructuresModifies(DateHeureDebObservation=self.date_observation)
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                result = [StructureModifieLP(**ele) for ele in tmp_data]
                self.found = (True, len(result))
            else:
                self.found = (False, 0)
                logger.debug('Aucun élément trouvé')
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

