from dataclasses import dataclass
from typing import List, Dict, Union,Optional
from wssiham.connection.zeepclient import Zeepclient
from wssiham.model.parametre.agent import ParamMatricule
from wssiham.model.response.siham_ws_ldap.recupUserIdResponse import Utilisateurs


import zeep
import logging


logger = logging.getLogger(__name__)


@dataclass
class RecupUserId(Zeepclient):
    """ L’objectif de ce flux est d’exposer tout ou une partie de la nomenclature RH d’un établissement
    donné aux applications externes.
    """

    liste_matricules: List[ParamMatricule]

    def response(self) -> Optional[Utilisateurs]:
        """ Récupère le login d'un matricule SIHAM

        Notes:
            La méthode RecupUserID du webservice utilisateurs de SIHAM permet de récupérer les informations ci-dessous :
                * donneesPersonnelles : Dict, None
                    * dateNaissance: datetime.datetime iso 8601
                    * matricule: str
                    * nomPatronymique: str
                    * nomUsuel: str
                    * numeroInseeDefinitif: str
                    * prenom: str
                    * userID: str, None
                * listeMatriculesErrones : Dict, None
                    * matriculeEnErreur : str
                    * messageErreur : str

        Returns:
            Liste de dictionnaire contenant le matricule SIHAM de l'utilisateur recherché ainsi que son login si disponible
        """
        result = None
        try:
            resp = \
                self.zeep_client.service.RecupUserID(ParamAgentid={"listeMatricules": self.liste_matricules})
            if resp:
                tmp_data = zeep.helpers.serialize_object(resp)
                donnees_personnelles = []
                matricules_errones = []

                for item in tmp_data:
                    if 'donneesPersonnelles' in item and item['donneesPersonnelles']:
                        donnees_personnelles.append(dict(item['donneesPersonnelles']))
                    if 'listeMatriculesErrones' in item and item['listeMatriculesErrones']:
                        matricules_errones.extend(item['listeMatriculesErrones'])
                result = Utilisateurs(listeDonneesPersonnelles=donnees_personnelles,
                                      listeMatriculesErrones=matricules_errones)
                logger.debug(result)
        except zeep.exceptions.Fault as fault:
            logger.error(fault)
        return result

        # result = None
        #
        # try:
        #     response = self.zeep_client.service.RecupUserID(
        #         ParamAgentid={"listeMatricules": self.liste_matricules}
        #     )
        #
        #     result = [DonneePersonnelle(**ele) for ele in tmp_data]
        #     for data in response:
        #         info = {}
        #         if data['donneesPersonnelles']:
        #             info['matricule'] = data['donneesPersonnelles']['matricule']
        #             info['login'] = data['donneesPersonnelles']['userID']
        #             info['error'] = False
        #         else:
        #             for error_matricule in data['listeMatriculesErrones']:
        #                 info['matricule'] = error_matricule['listeMatriculesErrones']['listeMatriculesErrones']['matriculeEnErreur']
        #                 info['login'] = None
        #                 info['error'] = True
        #         result.append(info)
        #     return result
        # except zeep.exceptions.Fault as fault:
        #     logger.error(fault)
        # return result

    # def __initialisation_object_liste(self):
    #     """ Paramètre pour l'utilisation du webservice RecupUserID
    #
    #     Returns:
    #         Un dictionnaire contenant les informations (paramètres) nécessaire pour l'utilisation
    #         du webservice RecupUserID
    #
    #     """
    #     t_lmatricule = self.zeep_client.get_type('ns0:Matricule')
    #     data = {'matcle': self.matcle}
    #     self.liste_matricule = t_lmatricule(**data)


